<?php
namespace Elementor;
use Elementor\Icons_Manager;

//If this file is called directly, abort.
if (!defined('ABSPATH')) {
    exit;   
}

//Class WooFoo Widget.
class WOOFOO_Widget extends Widget_Base {

    //Function for get the slug of the element name.
    public function get_name() {
        return 'woofoo';
    }

    //Function for get the name of the element.
    public function get_title() {
        return esc_html__('WooFoo', WOOFOO_DOMAIN);
    }

    //Function for get the icon of the element.
    public function get_icon() {
        return 'eicon-mega-menu';
    }

    //Function for include element into the category.
    public function get_categories() {
        return ['woofoo'];
    }

    //Adding the controls fields for WooFoo Element.
    protected function register_controls() {

        //Start WooFoo List General Section.
        $this->start_controls_section(
            'woofoo_general_section', array(
                'label'             => esc_html__('General', WOOFOO_DOMAIN),
            )
        );

        //Food Menu Type.
        $this->add_control(
            'woofoo_product_source', [
                'label'             => esc_html__('Food Menu Source', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SELECT,
                'options'           => woofoo_menu_type(),
                'default'           => 'custom',
            ]
        );

        //Food Menu Design.
        $this->add_control(
            'woofoo_product_layout', [
                'label'                 => esc_html__('Food Menu Layout', WOOFOO_DOMAIN),
                'type'                  => Controls_Manager::SELECT,
                'options'               => [
                    'woofoo-design-1'   => esc_html__('Design 1', WOOFOO_DOMAIN),
                    'woofoo-design-2'   => esc_html__('Design 2', WOOFOO_DOMAIN),
                    'woofoo-design-3'   => esc_html__('Design 3', WOOFOO_DOMAIN),
                    'woofoo-design-4'   => esc_html__('Design 4', WOOFOO_DOMAIN),
                    'woofoo-design-5'   => esc_html__('Design 5', WOOFOO_DOMAIN),
                    'woofoo-design-6'   => esc_html__('Design 6', WOOFOO_DOMAIN),
                    'woofoo-design-7'   => esc_html__('Design 7', WOOFOO_DOMAIN),
                    'woofoo-design-8'   => esc_html__('Design 8', WOOFOO_DOMAIN),
                    'woofoo-design-9'   => esc_html__('Design 9', WOOFOO_DOMAIN),
                    'woofoo-design-10'  => esc_html__('Design 10', WOOFOO_DOMAIN),
                    'woofoo-design-11'  => esc_html__('Design 11', WOOFOO_DOMAIN),
                ],
                'default'               => 'woofoo-design-1',
            ]
        );

        $this->end_controls_section();
        //End WooFoo List General Section.

        //Start Grid Options Section.
        $this->start_controls_section(
            'woofoo_grid_option_section', array(
                'label'             => esc_html__('Grid Options', WOOFOO_DOMAIN),
            )
        );

        //Grid Columns.
        $this->add_responsive_control(
            'woofoo_grid_columns', [
                'label'             => esc_html__('Grid Columns', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SELECT,
                'options'           => [
                    'default'       => 'Default',
                    '1'             => '1',
                    '2'             => '2',
                    '3'             => '3',
                    '4'             => '4',
                    '5'             => '5',
                    '6'             => '6',
                ],  
				'desktop_default'   => 'default',
				'tablet_default'    => 'default',
				'mobile_default'    => 'default',
                'prefix_class'      => 'elementor-grid%s-',
                'frontend_available'=> true,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-container' => 'grid-template-columns: repeat({{VALUE}},1fr);',
                ],
            ]
        );

        //Grid Column Gap.
        $this->add_responsive_control(
            'woofoo_product_column_gap', [
                'label'              => esc_html__('Grid Column Gap', WOOFOO_DOMAIN),
                'type'               => Controls_Manager::SLIDER,
                'default'            => [
                    'size'           => 30,
                ],      
                'range'              => [
                    'px'             => [
                        'min'        => 0,
                        'max'        => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'          => [
                    '{{WRAPPER}} .woofoo-container' => 'grid-column-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Grid Row Gap.
        $this->add_responsive_control(
            'woofoo_product_row_gap', [
                'label'             => esc_html__('Grid Row Gap', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'default'           => [
                    'size'          => 30,
                ],  
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-container' => 'grid-row-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Custom Currency Symbol.
        $this->add_control(
            'woofoo_currency_symbol', [
                'label'             => esc_html__('Currency Symbol', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SELECT,
                'options'           => [
                    ''              => esc_html__('None', WOOFOO_DOMAIN ),
                    'dollar'        => '&#36; ' . _x('Dollar', 'Currency', WOOFOO_DOMAIN),
                    'euro'          => '&#128; ' . _x('Euro', 'Currency', WOOFOO_DOMAIN),
                    'baht'          => '&#3647; ' . _x('Baht', 'Currency', WOOFOO_DOMAIN),
                    'franc'         => '&#8355; ' . _x('Franc', 'Currency', WOOFOO_DOMAIN),
                    'guilder'       => '&fnof; ' . _x('Guilder', 'Currency', WOOFOO_DOMAIN),
                    'krona'         => 'kr ' . _x('Krona', 'Currency', WOOFOO_DOMAIN),
                    'lira'          => '&#8356; ' . _x('Lira', 'Currency', WOOFOO_DOMAIN),
                    'peso'          => '&#8369; ' . _x('Peso', 'Currency', WOOFOO_DOMAIN),
                    'pound'         => '&#163; ' . _x('Pound Sterling', 'Currency', WOOFOO_DOMAIN),
                    'real'          => 'R$ ' . _x('Real', 'Currency', WOOFOO_DOMAIN),
                    'ruble'         => '&#8381; ' . _x('Ruble', 'Currency', WOOFOO_DOMAIN),
                    'rupee'         => '&#8360; ' . _x('Rupee', 'Currency', WOOFOO_DOMAIN),
                    'indian_rupee'  => '&#8377; ' . _x('Rupee (Indian)', 'Currency', WOOFOO_DOMAIN),
                    'shekel'        => '&#8362; ' . _x('Shekel', 'Currency', WOOFOO_DOMAIN),
                    'yen'           => '&#165; ' . _x('Yen/Yuan', 'Currency', WOOFOO_DOMAIN),
                    'won'           => '&#8361; ' . _x('Won', 'Currency', WOOFOO_DOMAIN),
                    'woofoo_custom' => esc_html__('Custom', WOOFOO_DOMAIN),
                ],
                'default'           => 'dollar',
                'condition'         => ['woofoo_product_source' => 'custom'],
            ]
        );

        //Custom Currency Text.
        $this->add_control(
            'woofoo_currency_text_symbol', [
                'label'             => esc_html__('Custom Symbol', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::TEXT,
                'condition'         => [
                    'woofoo_currency_symbol' => 'woofoo_custom',
                ],
            ]
        );

        $this->end_controls_section();
        //End Grid Options Section.

        //Start Custom Item Section.
        $this->start_controls_section(
            'woofoo_custom_item_section', array(
                'label'             => esc_html__('Item', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_product_source' => 'custom'],
            )
        );

        //Start Repeater Control.
        $repeater = new Repeater();

        //Custom Title.
        $repeater->add_control(
			'woofoo_title', [
                'label'             => esc_html__('Title', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::TEXT,
                'default'           => esc_html__('Title', WOOFOO_DOMAIN),
				'label_block'       => true,
                'dynamic'           => ['active' => true],
			]
		);

        //Custom Content.
        $repeater->add_control(
			'woofoo_list_content', [
				'label'             => esc_html__('Content', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::TEXTAREA,
                'default'           => esc_html__('Lorem ipsum dolor sit amet, consectetur adipisi cing elit sed do eiusmod', WOOFOO_DOMAIN),
			]
		);

        //Show Custom Image.
        $repeater->add_control(
            'woofoo_show_custom_image', [
                'label'             => esc_html__('Show Image', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
            ]
        );

        //Custom Image.
        $repeater->add_control(
			'woofoo_image', [
				'label'             => esc_html__('Choose Image', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::MEDIA,
                'dynamic'           => [
					'active'        => true,
				],
				'default'           => [
                    'url'           => WOOFOO_URL . 'assets/images/food-dish.png',
				],
                'condition'         => ['woofoo_show_custom_image' => 'yes'],
			]
		);

        //Show Custom Icon.
        $repeater->add_control(
            'woofoo_show_custom_icon', [
                'label'             => esc_html__('Show Icon', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'no',
            ]
        );

        //Custom Icon.
        $repeater->add_control(
            'woofoo_icon', [
                'label'             => esc_html__('Icon', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::ICONS,
                'default'           => [
                    'value'         => 'fas fa-leaf',
                    'library'       => 'fa-solid',
                ],
            ]
        );

        //Show Label.
        $repeater->add_control(
            'woofoo_show_label', [
                'label'             => esc_html__('Show Label', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
            ]
        );

        //Custom Label Text.
        $repeater->add_control(
			'woofoo_label', [
				'label'             => esc_html__('Label', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::TEXT,
				'default'           => esc_html__('Sale', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_show_label' => 'yes'],
			]
		);

        //Custom Regular Price.
        $repeater->add_control(
			'woofoo_regular_price', [
                'label'             => esc_html__('Regular Price', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::TEXT,
                'default'           => 12,
			]
		);

        //Custom Sale Price.
        $repeater->add_control(
			'woofoo_sale_price', [
                'label'             => esc_html__('Sale Price', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::TEXT,
                'default'           => 8,
			]
		);

        //Show Button.
        $repeater->add_control(
            'woofoo_show_btn', [
                'label'             => esc_html__('Show Button', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
            ]
        );

        //Custom Button Text.
        $repeater ->add_control(
			'woofoo_button', [
				'label'             => esc_html__('Button Text', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::TEXT,
				'dynamic'           => ['active' => true],
				'default'           => esc_html__('Order Now', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_show_btn' => 'yes'],
			]
		);

        //Button Link.
        $repeater->add_control(
			'woofoo_link', [
				'label'              => esc_html__('Link', WOOFOO_DOMAIN),
				'type'               => Controls_Manager::URL,
				'dynamic'            => [
					'active'         => true,
				],
				'placeholder'        => esc_html__('https://your-link.com', WOOFOO_DOMAIN),
				'default'            => [
					'url'            => '#',
				],
                'condition'         => ['woofoo_show_btn' => 'yes'],			
			]
		);

        //Show Customer Rating.
        $repeater->add_control(
            'woofoo_show_customer_rating', [
                'label'             => esc_html__('Show Customer Rating', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
            ]
        );

        //Custom Customer Rating.
        $repeater->add_control(
            'woofoo_customer_rating', [
                'label'             => esc_html__('Customer Rating', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SELECT,
                'options'           => [
                    '1'             => '1',
                    '2'             => '2',
                    '3'             => '3',
                    '4'             => '4',
                    '5'             => '5',
                ],
                'default'           => '3',
                'description'       => esc_html__('Select your rating.', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_show_customer_rating' => 'yes'],
            ]
        );

        //Show Highlight Product.
        $repeater->add_control(
            'woofoo_show_highlight', [
                'label'             => esc_html__('Show Highlight Product', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'no',
            ]
        );

        //Custom Repeater Item Default.
        $this->add_control(
			'woofoo_lists', [
				'label'             => esc_html__('List Item', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::REPEATER,
                'fields'            => $repeater->get_controls(),
                'default'           => [
                    [
                        'woofoo_title'   => esc_html__('List One', WOOFOO_DOMAIN),
                    ],
                    [
                        'woofoo_title'   => esc_html__('List Two', WOOFOO_DOMAIN),
                    ],
                    [
                        'woofoo_title'   => esc_html__('List Three', WOOFOO_DOMAIN),
                    ],
                ],
                'title_field'            => '{{{ woofoo_title }}}'
			]
        );   

        $this->end_controls_section();
        //End Custom Item Section.

        //Start WooCommerce Item Section.
        $this->start_controls_section(
            'woofoo_woo_item_section', array(
                'label'             => esc_html__('Item', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_product_source' => 'woocommerce'],
            )
        );

        //No. of Product.
        $this->add_control(
            'woofoo_number_of_products', [
                'label'             => esc_html__('Display No. of Product', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::NUMBER,
				'min'               => 1,
				'max'               => 100,
				'step'              => 1,
				'default'           => 10,
            ]
        );

        //WooCommerce Show Image.
        $this->add_control(
            'woofoo_show_image', [
                'label'             => esc_html__('Show Image', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
                'separator'         => 'before',
            ]
        );

        //WooCommerce Image Size.
        $this->add_group_control(
            Group_Control_Image_Size::get_type(), [
                'name'              => 'woofoo_product_image_size',
                'exclude'           => ['custom'],
                'default'           => 'medium',
                'condition'         => ['woofoo_show_image' => 'yes'],
            ]
        );

        //WooCommerce Show Title.
        $this->add_control(
            'woofoo_show_title', [
                'label'             => esc_html__('Show Title', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'separator'         => 'before',
                'default'           => 'yes',
            ]
        );

        //WooCommerce Show Description.
        $this->add_control(
            'woofoo_show_description', [
                'label'             => esc_html__('Show Description', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SELECT,
                'options'           => [
                    'none'          => esc_html__('None', WOOFOO_DOMAIN),
                    'short'         => esc_html__('Short', WOOFOO_DOMAIN),
					'full'          => esc_html__('Full', WOOFOO_DOMAIN),
                ],
                'default'           => 'short',
                'separator'         => 'after',
            ]
        );

        //WooCommerce Show Regular Price.
        $this->add_control(
            'woofoo_show_regular_price', [
                'label'             => esc_html__('Show Regular Price', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
            ]
        );

        //WooCommerce Show Sale Price.
        $this->add_control(
            'woofoo_show_sale_price', [
                'label'             => esc_html__('Show Sale Price', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
            ]
        );

        //WooCommerce Show Rating.
        $this->add_control(
            'woofoo_show_rating', [
                'label'             => esc_html__('Show Rating', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
            ]
        );

        //WooCommerce Sale Label Title.
        $this->add_control(
            'woofoo_sale', [
                'label'             => esc_html__('Sale', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::HEADING,
                'separator'         => 'before',
            ]   
        );

        //WooCommerce Show Sale Label.
        $this->add_control(
            'woofoo_show_label', [
                'label'             => esc_html__('Sale Label', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
                'separator'         => 'before',
            ]
        );

        //WooCommerce Sale Label Text.
        $this->add_control(
			'woofoo_sale_label', [
				'label'             => esc_html__('Sale Label Text', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::TEXT,
				'default'           => esc_html__('Sale', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_show_label' => 'yes'],
			]
		);

        //WooCommerce Button Label.
        $this->add_control(
            'woofoo_button', [
                'label'             => esc_html__('Button', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::HEADING,
                'separator'         => 'before',
            ]
        );

        //Show Cart Button.
        $this->add_control(
            'woofoo_show_cart_button', [
                'label'             => esc_html__('Show Cart Button', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
                'separator'         => 'before',
            ]
        );

        //Show Custom Button.
        $this->add_control(
            'woofoo_show_custom_button', [
                'label'             => esc_html__('Show Custom Button', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Show', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('Hide', WOOFOO_DOMAIN),
                'default'           => 'yes',
                'condition'         => ['woofoo_show_cart_button' => 'yes'],
            ]
        );

        //Simple Product Text.
        $this->add_control(
			'woofoo_simple_button_text', [
				'label'             => esc_html__('Simple Product Text', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::TEXT,
				'default'           => esc_html__('Order Now', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_show_custom_button' => 'yes'],
			]   
		);

        //Grouped Product Text.
        $this->add_control(
			'woofoo_grouped_button_text', [
				'label'             => esc_html__('Grouped Product Text', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::TEXT,
				'default'           => esc_html__('Order Now', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_show_custom_button' => 'yes'],
			]
		);

        //External/Affiliate Button Text.
        $this->add_control(
			'woofoo_external_button_text', [
				'label'             => esc_html__('External/Affiliate Button Text', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::TEXT,
				'default'           => esc_html__('Order Now', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_show_custom_button' => 'yes'],
			]
		);

        //Variable Product Text.
        $this->add_control(
			'woofoo_variable_button_text', [
				'label'             => esc_html__('Variable Product Text', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::TEXT,
				'default'           => esc_html__('Order Now', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_show_custom_button' => 'yes'],
			]
		);

        $this->end_controls_section();
        //End WooCommerce Item Section.

        //Start Products Query Section.
        $this->start_controls_section(
            'woofoo_product_query_section', array(
                'label'             => esc_html__('Products Query', WOOFOO_DOMAIN),
                'condition'         => ['woofoo_product_source' => 'woocommerce'],
            )
        );

        //Product Source.
        $this->add_control(
            'apcw_product_source', [
                'label'         => esc_html__('Product Source', WOOFOO_DOMAIN),
                'label_block'   => true,
                'type'          => Controls_Manager::SELECT,
                'default'       =>  'select',
                'multiple'      => true,
                'options'       => [
                    'select'                => esc_html__('Select', WOOFOO_DOMAIN ),
					'recent-products'       => esc_html__('Recent Products', WOOFOO_DOMAIN ),
					'featured-products'     => esc_html__('Featured Products', WOOFOO_DOMAIN ),
                    'best-selling-products' => esc_html__('Best Selling Products', WOOFOO_DOMAIN ),
                    'sale-products'         => esc_html__('Sale Products', WOOFOO_DOMAIN ),
                    'top-products'          => esc_html__('Top Rated Products', WOOFOO_DOMAIN ),
				],
            ]
        );

        //Include Categories.
        $this->add_control(
            'woofoo_product_categories', [
                'label'             => esc_html__('Product Categories', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SELECT2,
                'label_block'       => true,
                'multiple'          => true,
                'description'       => esc_html__('Select the categories you want to show', WOOFOO_DOMAIN),
                'separator'         => 'before',
                'options'           => woofoo_product_categories(),
            ]
        );

        //Exclude Categories.
        $this->add_control(
            'woofoo_exclude_product_categories', [
                'label'             => esc_html__('Exclude Above Categories', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Yes', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('No', WOOFOO_DOMAIN),
                'default'           => 'yes',
            ]
        );

        //Include Tags.
        $this->add_control(
            'woofoo_product_tags', [
                'label'             => esc_html__('Product Tags', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SELECT2,
                'label_block'       => true,
                'multiple'          => true,
                'description'       => esc_html__('Select the tags you want to show', WOOFOO_DOMAIN),
                'options'           => woofoo_product_tags(),
                'separator'         => 'before',
            ]
        );

        //Exclude Tags.
        $this->add_control(
            'woofoo_exclude_product_tags', [
                'label'             => esc_html__('Exclude Above Tags', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SWITCHER,
                'label_on'          => esc_html__('Yes', WOOFOO_DOMAIN),
                'label_off'         => esc_html__('No', WOOFOO_DOMAIN),
                'default'           => 'yes',
            ]
        );

        //Product Order By.
        $this->add_control(
            'woofoo_product_orderby', [
                'label'             => esc_html__('Order By', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SELECT,
                'default'           => 'date',
                'options'           => [
                'date'              => esc_html__('Date', WOOFOO_DOMAIN),
                'title'             => esc_html__('Title', WOOFOO_DOMAIN),
                'menu_order'        => esc_html__('Menu Order', WOOFOO_DOMAIN),
                'rand'              => esc_html__('Random', WOOFOO_DOMAIN),
                'ID'                => esc_html__('Post ID', WOOFOO_DOMAIN),
                'author'            => esc_html__('Author', WOOFOO_DOMAIN),
                'rating'            => esc_html__('Rating', WOOFOO_DOMAIN),
                ],
            ]
        );

        //Product Sort By.
        $this->add_control(
            'woofoo_product_sort_order', [
                'label'             => esc_html__('Sort By', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SELECT,
                'default'           => 'desc',
                'options'           => [
                    'asc'           => esc_html__('ASC', WOOFOO_DOMAIN),
                    'desc'          => esc_html__('DESC', WOOFOO_DOMAIN),
                ],
            ]
        );

        $this->end_controls_section();
        //End Products Query Section.
        //End Content Section

        //Style Section Start.
        //Start Item Style.
        $this->start_controls_section(
            'woofoo_item_style', [
                'label'             => esc_html__('Item', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        //Item Background Color.
        $this->add_control( 
            'woofoo_item_bg_color', [
                'label'             => esc_html__('Item Background Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-1 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-2 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-3 .woofoo-content,
                    {{WRAPPER}} .woofoo-design-4 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-5 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-6 .woofoo-content,
                    {{WRAPPER}} .woofoo-design-7 .woofoo-content,
                    {{WRAPPER}} .woofoo-design-8 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-9 .woofoo-content-wrapper,
                    {{WRAPPER}} .woofoo-design-10 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-11 .woofoo-content' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        //Item Highlight Background Color.
        $this->add_control( 
            'woofoo_highlight_bg_color', [
                'label'             => esc_html__('Highlight Background Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-1 .woofoo-item-container.woofoo-highlight,
                    {{WRAPPER}} .woofoo-design-2 .woofoo-item-container.woofoo-highlight,
                    {{WRAPPER}} .woofoo-design-3 .woofoo-item-container.woofoo-highlight .woofoo-content,
                    {{WRAPPER}} .woofoo-design-4 .woofoo-item-container.woofoo-highlight,
                    {{WRAPPER}} .woofoo-design-5 .woofoo-item-container.woofoo-highlight,
                    {{WRAPPER}} .woofoo-design-6 .woofoo-item-container.woofoo-highlight .woofoo-content,
                    {{WRAPPER}} .woofoo-design-7 .woofoo-item-container.woofoo-highlight .woofoo-content,
                    {{WRAPPER}} .woofoo-design-8 .woofoo-item-container.woofoo-highlight,
                    {{WRAPPER}} .woofoo-design-9 .woofoo-item-container.woofoo-highlight .woofoo-content-wrapper,
                    {{WRAPPER}} .woofoo-design-10 .woofoo-item-container.woofoo-highlight,
                    {{WRAPPER}} .woofoo-design-11 .woofoo-item-container.woofoo-highlight .woofoo-content' => 'background-color: {{VALUE}} !important;',
                ],
                'condition'         => ['woofoo_product_source' => 'custom'],
            ]
        );

        //Item Highlight Border Color.
        $this->add_control(
            'woofoo_highlight_border_color', [
                'label'             => esc_html__('Highlight Border Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'default'           => '#ea1f2e',
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container.woofoo-highlight,
                    {{WRAPPER}} .woofoo-design-9 .woofoo-item-container.woofoo-highlight .woofoo-content-wrapper,
                    {{WRAPPER}} .woofoo-design-11 .woofoo-item-container.woofoo-highlight .woofoo-content' => 'border-color: {{VALUE}} !important;',
                    '{{WRAPPER}} .woofoo-design-10 .woofoo-highlight.woofoo-item-container::before' => 'background-color: {{VALUE}} !important;',
                ],
                'condition'         => ['woofoo_product_source' => 'custom'],
            ]
        );

        //Item Divider Color.
        $this->add_control(
            'woofoo_divider_color', [
                'label'             => esc_html__('Divider Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-10 .woofoo-item-container::before' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} .woofoo-design-10 .woofoo-item-container' => 'border-top-color: {{VALUE}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-10'],
                ]
            ]
        );

        //Item Divider Width.
        $this->add_control(
			'woofoo_devider-width', [
                'label'             => esc_html__('Item Divider Width', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::SLIDER,
				'size_units'        => [ 'px' ],
				'range'             => [
					'px'            => [
						'min'       => 0,
						'max'       => 10,
						'step'      => 1,
					],
				],
				'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-10 .woofoo-item-container::before' => 'width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .woofoo-design-10 .woofoo-item-container' => 'border-top-width: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-10'],
                ]
			]
		);

        //Item Border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woofoo_item_border',
                'selector'          => '{{WRAPPER}} .woofoo-design-1 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-2 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-3 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-4 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-5 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-6 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-7 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-8 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-9 .woofoo-content-wrapper,
                {{WRAPPER}} .woofoo-design-10 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-11 .woofoo-content',
            ]
        );

        //Item Box Shadow.
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(), [
                'name'              => 'woofoo_box_shadow',
                'selector'          => '{{WRAPPER}} .woofoo-design-1 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-2 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-3 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-4 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-5 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-6 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-7 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-8 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-9 .woofoo-content-wrapper,
                {{WRAPPER}} .woofoo-design-10 .woofoo-item-container,
                {{WRAPPER}} .woofoo-design-11 .woofoo-content',
            ]
        );

        //Item Border Radius.
        $this->add_responsive_control(
            'woofoo_item_border_radius', [
                'label'             => esc_html__('Item Border Radius', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-1 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-2 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-3 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-4 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-5 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-6 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-7 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-7 .woofoo-content,
                    {{WRAPPER}} .woofoo-design-8 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-9 .woofoo-content-wrapper,
                    {{WRAPPER}} .woofoo-design-10 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-11 .woofoo-content' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        //Item Padding.
        $this->add_responsive_control(
            'woofoo_item_padding', [
                'label'             => esc_html__('Item Padding', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-5 .woofoo-item-container,
                    {{WRAPPER}} .woofoo-design-8 .woofoo-item-container' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-5', 'woofoo-design-8'],
                ]
            ]
        );

        //Item Content Padding.
        $this->add_responsive_control(
            'woofoo_content_padding', [
                'label'             => esc_html__('Content Padding', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-content' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-1', 'woofoo-design-2', 'woofoo-design-3', 'woofoo-design-4', 'woofoo-design-5', 'woofoo-design-6', 'woofoo-design-7','woofoo-design-8','woofoo-design-10','woofoo-design-11'],
                ]
            ]
        );

        //Item Content Padding.
        $this->add_responsive_control(
            'woofoo_content_wrapper_padding', [
                'label'             => esc_html__('Content Padding', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'default'           => [
                    'top'           => '20',
                    'right'         => '20',
                    'bottom'        => '20',
                    'left'          => '20',
                    'isLinked'      => true,
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-content-wrapper' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-9'],
                ]
            ]
        );

        //Content Alignment.
        $this->add_responsive_control(
            'woofoo_content_align', [
                'label'             => esc_html__('Alignment', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::CHOOSE,
                'options'           => [
                    'left'          => [
                        'title'     => esc_html__('Left', WOOFOO_DOMAIN),
                        'icon'      => 'eicon-text-align-left',
                    ],
                    'center'        => [
                        'title'     => esc_html__('Center', WOOFOO_DOMAIN),
                        'icon'      => 'eicon-text-align-center',
                    ],
                    'right'         => [
                        'title'     => esc_html__('Right', WOOFOO_DOMAIN),
                        'icon'      => 'eicon-text-align-right',
                    ],
                ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-content' => 'text-align: {{VALUE}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-2', 'woofoo-design-3', 'woofoo-design-4', 'woofoo-design-6', 'woofoo-design-7','woofoo-design-8', 'woofoo-design-11'],
                ]
            ]
        );

        $this->end_controls_section();
        //End Item Style.

        //Start Category Style.
        $this->start_controls_section(
            'woofoo_category_style', [
                'label'             => esc_html__('Category', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
                'condition'         => ['woofoo_product_source' => 'woocommerce'],
            ]
        );

        //Category Typography.
        $this->add_group_control(
			Group_Control_Typography::get_type(), [
				'name'              => 'category_typography',
				'selector'          => '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-category',
			]
		);

        //Category Color.
        $this->add_control( 
            'woofoo_category_color', [
                'label'             => esc_html__('Category Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-category' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Category Background Color.
        $this->add_control( 
            'woofoo_category_bg_color', [
                'label'             => esc_html__('Background Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-category' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        //Category Border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woofoo_category_border',
                'selector'          => '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-category',
            ]
        );

        //Category Border Radius.
        $this->add_responsive_control(
            'woofoo_category_border_radius', [
                'label'             => esc_html__('Category Border Radius', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-category' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        //Category Padding.
        $this->add_responsive_control(
            'woofoo_category_padding', [
                'label'             => esc_html__('Padding', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-category' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        //Category Spacing.
        $this->add_responsive_control(
            'woofoo_category_spacing', [
                'label'             => esc_html__('Category Spacing', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-category' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Category Style.

        //Start Tag Style.
        $this->start_controls_section(
            'woofoo_tag_style', [
                'label'             => esc_html__('Tag', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
                'condition'         => ['woofoo_product_source' => 'woocommerce'],
            ]
        );

        //Tag Typography.
        $this->add_group_control(
			Group_Control_Typography::get_type(), [
				'name'              => 'woofoo_tag_typography',
				'selector'          => '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-tag',
			]
		);

        //Tag Color.
        $this->add_control( 
            'woofoo_tag_color', [
                'label'             => esc_html__('Tag Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-tag' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Tag Background Color.
        $this->add_control( 
            'woofoo_tag_bg_color', [
                'label'             => esc_html__('Background Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-tag' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        //Tag Border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woofoo_tag_border',
                'selector'          => '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-tag',
            ]
        );

        //Tag Border Radius.
        $this->add_responsive_control(
            'woofoo_tag_border_radius', [
                'label'             => esc_html__('Tag Border Radius', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-tag' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        //Tag Padding.
        $this->add_responsive_control(
            'woofoo_tag_padding', [
                'label'             => esc_html__('Padding', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-tag' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        //Tag Spacing.
        $this->add_responsive_control(
            'woofoo_tag_spacing', [
                'label'             => esc_html__('Tag Spacing', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'          => [
                    '{{WRAPPER}} .woofoo-cat-tag-title .woofoo-tag' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Tag Style.

        //Start Image Style.
        $this->start_controls_section(
            'woofoo_image_style', [
                'label'             => esc_html__('Image', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        //Image Border Radius.
        $this->add_responsive_control(
            'woofoo_image_border_radius', [
                'label'             => esc_html__('Image Border Radius', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-image img,
                    {{WRAPPER}} .woofoo-woocommerce .woofoo-img img,
                    {{WRAPPER}} .woofoo-design-1 .woofoo-img-icon-wrapper,
                    {{WRAPPER}} .woofoo-design-2 .woofoo-item-container:hover .woofoo-image:after,
                    {{WRAPPER}} .woofoo-design-2 .woofoo-item-container:hover .woofoo-layout-img:after' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-1', 'woofoo-design-2','woofoo-design-4', 'woofoo-design-5', 'woofoo-design-7','woofoo-design-9', 'woofoo-design-11'],
                ]
            ]
        );

        //Image Border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woofoo_image_border',
                'selector'          => '{{WRAPPER}} .woofoo-design-2 .woofoo-image img,
                {{WRAPPER}} .woofoo-design-3 .woofoo-image img,
                {{WRAPPER}} .woofoo-design-4 .woofoo-image img,
                {{WRAPPER}} .woofoo-design-5 .woofoo-image img,
                {{WRAPPER}} .woofoo-design-6 .woofoo-image img,
                {{WRAPPER}} .woofoo-design-7 .woofoo-image img,
                {{WRAPPER}} .woofoo-design-8 .woofoo-item-container .woofoo-img-wrapper,
                {{WRAPPER}} .woofoo-design-9 .woofoo-image img,
                {{WRAPPER}} .woofoo-design-11 .woofoo-image img,
                {{WRAPPER}} .woofoo-design-2 .woofoo-layout-img img,
                {{WRAPPER}} .woofoo-design-3 .woofoo-layout-img img,
                {{WRAPPER}} .woofoo-design-4 .woofoo-layout-img img,
                {{WRAPPER}} .woofoo-design-5 .woofoo-layout-img img,
                {{WRAPPER}} .woofoo-design-6 .woofoo-layout-img img,
                {{WRAPPER}} .woofoo-design-7 .woofoo-layout-img img,
                {{WRAPPER}} .woofoo-design-8 .woofoo-layout-img,
                {{WRAPPER}} .woofoo-design-9 .woofoo-layout-img img,
                {{WRAPPER}} .woofoo-design-11 .woofoo-layout-img img',
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-2', 'woofoo-design-3', 'woofoo-design-4', 'woofoo-design-5', 'woofoo-design-6', 'woofoo-design-7', 'woofoo-design-8','woofoo-design-9', 'woofoo-design-10', 'woofoo-design-11'],
                ]
            ]
        );

        //Image Padding.
        $this->add_responsive_control(
            'woofoo_image_padding', [
                'label'             => esc_html__('Padding', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-4 .woofoo-image,
                    {{WRAPPER}} .woofoo-design-10 .woofoo-img-icon-wrapper' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-4','woofoo-design-10'],
                ],
            ]   
        );

        //Image Width.
        $this->add_responsive_control(
            'woofoo_image_width', [
                'label'             => esc_html__('Image Width', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => [ 'px', '%' ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 400,
                    ],
                    '%' => [
						'min' => 0,
						'max' => 100,
					],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-11 .woofoo-layout-img .woofoo-image img,
                    {{WRAPPER}} .woofoo-design-11 .woofoo-image img,
                    {{WRAPPER}} .woofoo-design-8 .woofoo-img-wrapper,
                    {{WRAPPER}} .woofoo-design-8 .woofoo-woocommerce .woofoo-layout-img,
                    {{WRAPPER}} .woofoo-design-9 .woofoo-img-icon-wrapper,
                    {{WRAPPER}} .woofoo-design-9 .woofoo-layout-img'  => 'max-width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .woofoo-item-container .woofoo-img-icon-wrapper,
                    {{WRAPPER}} .woofoo-item-container .woofoo-woocommerce .woofoo-layout-img,
                    {{WRAPPER}} .woofoo-item-container .woofoo-woocommerce .woofoo-layout-img img,
                    {{WRAPPER}} .woofoo-item-container .woofoo-img-icon-wrapper img,
                    {{WRAPPER}} .woofoo-design-4 .woofoo-image img,
                    {{WRAPPER}} .woofoo-design-4 .woofoo-woocommerce .woofoo-img img,
                    {{WRAPPER}} .woofoo-design-7 .woofoo-image img,
                    {{WRAPPER}} .woofoo-design-7 .woofoo-woocommerce .woofoo-img img'  => 'width: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-1', 'woofoo-design-2', 'woofoo-design-4', 'woofoo-design-5','woofoo-design-7','woofoo-design-8', 'woofoo-design-9', 'woofoo-design-10', 'woofoo-design-11'],
                ]
            ]
        );

        //Image Width.
        $this->add_responsive_control(
            'woofoo_image_height', [
                'label'             => esc_html__('Image Height', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'size_units'        => [ 'px', '%' ],
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 400,
                    ],
                    '%' => [
						'min' => 0,
						'max' => 100,
					],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-img-wrapper,
                    {{WRAPPER}} .woofoo-item-container .woofoo-woocommerce .woofoo-layout-img' => 'height: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .woofoo-design-11 .woofoo-layout-img .woofoo-image img,
                    {{WRAPPER}} .woofoo-design-11 .woofoo-image img'  => 'height: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .woofoo-item-container .woofoo-img-icon-wrapper,
                    {{WRAPPER}} .woofoo-item-container .woofoo-woocommerce .woofoo-layout-img,
                    {{WRAPPER}} .woofoo-item-container .woofoo-woocommerce .woofoo-layout-img img,
                    {{WRAPPER}} .woofoo-item-container .woofoo-img-icon-wrapper img,
                    {{WRAPPER}} .woofoo-design-4 .woofoo-image img,
                    {{WRAPPER}} .woofoo-design-4 .woofoo-woocommerce .woofoo-img img,
                    {{WRAPPER}} .woofoo-design-7 .woofoo-image img,
                    {{WRAPPER}} .woofoo-design-7 .woofoo-woocommerce .woofoo-img img'  => 'height: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-1', 'woofoo-design-2', 'woofoo-design-4', 'woofoo-design-5','woofoo-design-7','woofoo-design-8', 'woofoo-design-9', 'woofoo-design-10', 'woofoo-design-11'],
                ]
            ]
        );

        $this->end_controls_section();
        //End Box Style.

        //Start Title Style.
        $this->start_controls_section(
            'woofoo_title_style', [
                'label'             => esc_html__('Title', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        //Title Typography.
        $this->add_group_control(
			Group_Control_Typography::get_type(), [
				'name'              => 'woofoo_title_typography',
				'selector'          => '{{WRAPPER}} .woofoo-item-container.woofoo-woocommerce .woofoo-title a,
                {{WRAPPER}} .woofoo-item-container .woofoo-title',
			]
		);

        //Title Color.
        $this->add_control( 
            'woofoo_title_color', [
                'label'             => esc_html__('Title Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container.woofoo-woocommerce .woofoo-title a,
                {{WRAPPER}} .woofoo-item-container .woofoo-title' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Title Spacing.
        $this->add_responsive_control(
            'woofoo_title_spacing', [
                'label'             => esc_html__('Title Spacing', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-title-price-wrapper,
                    {{WRAPPER}} .woofoo-design-5 .woofoo-title-wrapper,
                    {{WRAPPER}} .woofoo-design-7 .woofoo-title-wrapper' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-1', 'woofoo-design-2', 'woofoo-design-3', 'woofoo-design-4', 'woofoo-design-6', 'woofoo-design-7','woofoo-design-8'],
                ]
            ]
        );

        //Title Spacing.
        $this->add_responsive_control(
            'woofoo_title_odd_spacing', [
                'label'             => esc_html__('Title Spacing (odd)', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 200,
                    ],
                ],
                'default'   => [
					'unit'  => 'px',
					'size'  => 20,
				],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-container .woofoo-design-9:nth-child(odd) .woofoo-title-wrapper ' => 'margin-left: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-9'],
                ]
            ]
        );

        //Title Spacing.
        $this->add_responsive_control(
            'woofoo_title_even_spacing', [
                'label'             => esc_html__('Title Spacing (even)', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 200,
                    ],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-container .woofoo-design-9:nth-child(even) .woofoo-title-wrapper ' => 'margin-left: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-9'],
                ]
            ]
        );

        //Dotted Separator Color.
        $this->add_control( 
            'woofoo_separator_color', [
                'label'             => esc_html__('Dotted Separator Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-separator' => 'border-color: {{VALUE}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-1', 'woofoo-design-2', 'woofoo-design-3', 'woofoo-design-4', 'woofoo-design-6', 'woofoo-design-8' ],
                ]
            ]
        );

        $this->end_controls_section();
        //End Title Style.

        //Start Description Style.
        $this->start_controls_section(
            'woofoo_description_style', [
                'label'             => esc_html__('Description', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        //Description Typography.
        $this->add_group_control(
			Group_Control_Typography::get_type(), [
				'name'              => 'woofoo_description_typography',
				'selector'          => '{{WRAPPER}} .woofoo-item-container .woofoo-description p',
			]
		);

        //Description Text Color.
        $this->add_control( 
            'woofoo_description_color', [
                'label'             => esc_html__('Description Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-description p' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Description Spacing.
        $this->add_responsive_control(
            'woofoo_description_spacing', [
                'label'             => esc_html__('Description Spacing', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'          => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-description' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-1', 'woofoo-design-2', 'woofoo-design-3', 'woofoo-design-4', 'woofoo-design-6', 'woofoo-design-7', 'woofoo-design-8', 'woofoo-design-9', 'woofoo-design-10', 'woofoo-design-11'],
                ]
            ]
        );

        //Description Spacing.
        $this->add_responsive_control(
            'woofoo_description_style_spacing', [
                'label'             => esc_html__('Description Spacing', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'          => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-description p' => 'margin-left: {{SIZE}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-5' ],
                ]
            ]
        );

        $this->end_controls_section();
        //End Description Style.

        //Start Price Style.
        $this->start_controls_section(
            'woofoo_price_style', [
                'label'             => esc_html__('Price', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        //Currency Symbol Position.
        $this->add_control(
			'woofoo_currency_position', [
				'label'             => esc_html__('Currency Symbol Position', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::CHOOSE,
				'default'           => 'before',
				'options'           => [
					'before'        => [
						'title'     => esc_html__('Before', WOOFOO_DOMAIN),
						'icon'      => 'eicon-h-align-left',
					],
					'after'         => [
						'title'     => esc_html__('After', WOOFOO_DOMAIN),
						'icon'      => 'eicon-h-align-right',
					],
				],
                'condition'         => ['woofoo_product_source' => 'custom'],
			]
		);

        //Regular Price Typography.
        $this->add_group_control(
			Group_Control_Typography::get_type(), [
				'name'              => 'woofoo_regular_price_typography',
                'label'             => esc_html__('Regular Price Typography', WOOFOO_DOMAIN),
				'selector'          => '{{WRAPPER}} .woofoo-item-container .woofoo-regular-price',
			]
		);

        //Regular Price Color.
        $this->add_control( 
            'woofoo_regular_color', [
                'label'             => esc_html__('Regular Price Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-regular-price' => 'color: {{VALUE}};',
                ],
            ]
        );

        // Price Color.
		$this->add_control(
			'woofoo_price_line_color', [
				'label'         	=> esc_html__('Price Line Through', WOOFOO_DOMAIN),
				'type'          	=> Controls_Manager::COLOR,
				'selectors'     	=> [
					'{{WRAPPER}} .woofoo-item-container .woofoo-line' => 'text-decoration-color: {{VALUE}};',
				],
			]
		);

		// Title Spacing.
		$this->add_responsive_control(
			'woofoo_line_width', [
				'label' 			=> esc_html__('Price Line Through Width', WOOFOO_DOMAIN),
				'type' 				=> Controls_Manager::SLIDER,
				'size_units' 		=> ['px'],
				'range'				=> [
					'px' 			=> [
						'min' 		=> 0,
						'max' 		=> 10,
						'step' 		=> 1,
					],
				],
				'default' 			=> [
					'unit' 			=> 'px',
					'size' 			=> 1,
				],
				'selectors' 		=> [
					'{{WRAPPER}} .woofoo-item-container .woofoo-line' => 'text-decoration-thickness: {{SIZE}}{{UNIT}};',
				],
			]
		);

        //Sale Price Typography.
        $this->add_group_control(
			Group_Control_Typography::get_type(), [
				'name'              => 'woofoo_sale_price_typography',
                'label'             => esc_html__('Sale Price Typography', WOOFOO_DOMAIN),
				'selector'          => '{{WRAPPER}} .woofoo-item-container .woofoo-sale-price',
			]
		);

        //Sale Price Color.
        $this->add_control( 
            'woofoo_sale_color', [
                'label'             => esc_html__('Sale Price Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-sale-price' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Price Background Color.
        $this->add_control( 
            'woofoo_price_bg_color', [
                'label'             => esc_html__('Background Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'default'           => '#FFFFFF',
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-11 .woofoo-all-price' => 'background-color: {{VALUE}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-11'],
                ]
            ]
        );

        //Price Border Radius.
        $this->add_responsive_control(
            'woofoo_price_border_radius', [
                'label'             => esc_html__('Border Radius', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-11 .woofoo-all-price' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-11'],
                ]
            ]
        );

        //Price Padding.
        $this->add_responsive_control(
            'woofoo_price_padding', [
                'label'             => esc_html__('Padding', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-design-11 .woofoo-all-price' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-11'],
                ]
            ]
        );

        //Price border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woofoo_price_border',
                'selector'          => '{{WRAPPER}} .woofoo-design-11 .woofoo-all-price',
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-11'],
                ]
            ]
        );

        //Price Spacing.
        $this->add_responsive_control(
            'woofoo_price_spacing', [
                'label'             => esc_html__('Price Spacing', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'default'           => [
					'unit'          => 'px',
					'size'          => 5,
				],
                'frontend_available' => true,
                'selectors'          => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-sale-price' => 'margin-right: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Price Style.

        //Start Button Style.
        $this->start_controls_section(
            'woofoo_button_style', [
                'label'             => esc_html__('Button', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        //Button Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'              => 'woofoo_button_typography',
                'selector'          => '{{WRAPPER}} .woofoo-item-container .woofoo-button-text',
            ]
        );

        //Button Spacing.
        $this->add_responsive_control(
            'woofoo_button_spacing', [
                'label'             => esc_html__('Button Spacing', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'          => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-button' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Start Button Tab.
        $this->start_controls_tabs('woofoo_button_color_tabs');

        //Start Normal Button Tab.
        $this->start_controls_tab('woofoo_normal_button', [
            'label'                 => esc_html__('Normal', WOOFOO_DOMAIN),
            ]
        );

        //Button Text Color.
        $this->add_control(
            'woofoo_button_color', [
                'label'             => esc_html__('Text Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-button-text' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Button Background Color.
        $this->add_control(
            'woofoo_button_bg_color', [
                'label'             => esc_html__('Button Background Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-button .woofoo-button-text' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Normal Button Tab.

        //Start Hover Button Tab.
        $this->start_controls_tab('woofoo_hover_button', [
            'label'                 => esc_html__('Hover', WOOFOO_DOMAIN),
            ]
        );

        //Button Text Color.
        $this->add_control(
            'woofoo_button_color_hover', [
                'label'             => esc_html__('Text Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-button .woofoo-button-text:hover' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Button Background Color.
        $this->add_control(
            'woofoo_button_bg_color_hover', [
                'label'             => esc_html__('Button Background Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-button .woofoo-button-text:hover' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        //Button border.
        $this->add_control(
            'woofoo_border_color_hover', [
                'label'             => esc_html__('Border Hover Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-button .woofoo-button-text:hover' => 'border-color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Hover Button Tab.

        $this->end_controls_tabs();
        //End Button Tab.

        //Button border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woofoo_button_border',
                'selector'          => '{{WRAPPER}} .woofoo-item-container .woofoo-button .woofoo-button-text',
            ]
        );

        //Button Border Radius.
        $this->add_responsive_control(
            'woofoo_border_radius', [
                'label'             => esc_html__('Border Radius', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-button .woofoo-button-text' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        //Button Border Radius.
        $this->add_responsive_control(
            'woofoo_button_padding', [
                'label'             => esc_html__('Button Padding', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-button .woofoo-button-text' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Button Style.

        //Start Icon Style.
        $this->start_controls_section(
            'woofoo_icon_style', [
                'label'             => esc_html__('Icon', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        //Icon Size.
        $this->add_responsive_control(
			'woofoo_icon_size', [
				'label'             => esc_html__('Icon Size', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::SLIDER,
				'size_units'        => [ 'px', '%' ],
				'range'             => [
					'px'            => [
						'min'       => 0,
						'max'       => 100,
						'step'      => 1,
					],
				],
                'default'           => [
                    'size'          => 14,
					'unit'          => 'px',
                ],
				'selectors'         => [
					'{{WRAPPER}} .woofoo-item-container .woofoo-icon i' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .woofoo-item-container .woofoo-icon svg' => 'width:{{SIZE}}{{UNIT}}; height:{{SIZE}}{{UNIT}};',
				],
			]
		);

        //Icon Color.
        $this->add_control(
			'woofoo_icon_color', [
				'label'             => esc_html__('Icon Color', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::COLOR,
                'default'           => '#ffffff',
				'selectors'         => [
					'{{WRAPPER}} .woofoo-item-container .woofoo-icon i' => 'color: {{VALUE}};',
                    '{{WRAPPER}} .woofoo-item-container .woofoo-icon svg' => 'fill:{{VALUE}}',
				],
			]
		);

        //Icon Background Color.
        $this->add_control(
			'woofoo_icon_background_color', [
				'label'             => esc_html__('Icon Background Color', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::COLOR,
                'default'           => '#ea1f2e',
				'selectors'         => [
					'{{WRAPPER}} .woofoo-item-container .woofoo-icon' => 'background-color: {{VALUE}} !important;',
				],
			]
		);

        //Icon border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woofoo_icon_border',
                'selector'          => '{{WRAPPER}} .woofoo-item-container .woofoo-icon',
            ]
        );

        //Icon Border Radius.
        $this->add_responsive_control(
            'woofoo_icon_border_radius', [
                'label'             => esc_html__('Icon Border Radius', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-icon' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        //Icon Padding.
        $this->add_responsive_control(
            'woofoo_icon_padding', [
                'label'             => esc_html__('Icon Padding', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-icon' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Icon Style.

        //Start Rating Style.
        $this->start_controls_section(
            'woofoo_rating_style', [
                'label'             => esc_html__('Rating', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        //Rating Spacing.
        $this->add_responsive_control(
            'woofoo_rating_spacing', [
                'label'             => esc_html__('Rating Spacing', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::SLIDER,
                'range'             => [
                    'px'            => [
                        'min'       => 0,
                        'max'       => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-rating,
                    {{WRAPPER}} .woofoo-woocommerce .woofoo-star-rating' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Rating Size.
        $this->add_control(
			'woofoo_rating_size', [
				'label'             => esc_html__('Rating Size', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::SLIDER,
				'size_units'        => [ 'px', '%' ],
				'range'             => [
					'px'            => [
						'min'       => 0,
						'max'       => 100,
						'step'      => 1,
					],
				],
                'default'           => [
                    'size'          => 14,
					'unit'          => 'px',
                ],
				'selectors'         => [
					'{{WRAPPER}} .woofoo-item-container .woofoo-rating i,
                    {{WRAPPER}} .woofoo-woocommerce .woofoo-star-rating' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);

        //Rating Color.
        $this->add_control(
			'woofoo_rating_color', [
				'label'             => esc_html__('Rating Color', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::COLOR,
				'selectors'         => [
					'{{WRAPPER}} .woofoo-item-container .woofoo-rating .star,
                    {{WRAPPER}} .woofoo-woocommerce .woofoo-star-rating::before' => 'color: {{VALUE}};',
				],
			]
		);

        //Rating Fill Color.
        $this->add_control(
			'woofoo_rating_fill_color', [
				'label'             => esc_html__('Rating Fill Color', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::COLOR,
				'selectors'         => [
					'{{WRAPPER}} .woofoo-item-container .woofoo-rating .star-fill,
                    {{WRAPPER}} .woofoo-woocommerce .woofoo-star-rating span' => 'color: {{VALUE}};',
				],
			]
		);

        $this->end_controls_section();
        //End Rating Style.

        //Start Label Style.
        $this->start_controls_section(
            'woofoo_label_style', [
                'label'             => esc_html__('Label', WOOFOO_DOMAIN),
                'tab'               => Controls_Manager::TAB_STYLE,
            ]
        );

        //Label Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'              => 'woofoo_label_typography',
                'selector'          => '{{WRAPPER}} .woofoo-item-container .woofoo-label,
                {{WRAPPER}} .woofoo-design-6 .woofoo-label',
            ]
        );

        //Label Color.
        $this->add_control( 
            'woofoo_label_color', [
                'label'             => esc_html__('Label Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-label,
                    {{WRAPPER}} .woofoo-design-6 .woofoo-label' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Label Background Color.
        $this->add_control( 
            'woofoo_label_bg_color', [
                'label'             => esc_html__('Label Background Color', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::COLOR,
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-label,
                    {{WRAPPER}} .woofoo-design-3 .woofoo-label::after,
                    {{WRAPPER}} .woofoo-design-6 .woofoo-label,
                    {{WRAPPER}} .woofoo-design-11 .woofoo-label::after' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        //Label Border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'              => 'woofoo_label_border',
                'selector'          => '{{WRAPPER}} .woofoo-item-container .woofoo-label',
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-1', 'woofoo-design-9'],
                ]
            ]
        );

        //Label Border Radius.
        $this->add_responsive_control(
            'woofoo_label_border_radius', [
                'label'             => esc_html__('Label Border Radius', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-label' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout' => ['woofoo-design-1', 'woofoo-design-9'],
                ]
            ]
        );

        //Label Padding.
        $this->add_responsive_control(
            'woofoo_label_padding', [
                'label'             => esc_html__('Label Padding', WOOFOO_DOMAIN),
                'type'              => Controls_Manager::DIMENSIONS,
                'size_units'        => [ 'px', '%' ],
                'selectors'         => [
                    '{{WRAPPER}} .woofoo-item-container .woofoo-label' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'         => [
                    'woofoo_product_layout!' => 'woofoo-design-6',
                    'woofoo_product_layout' => ['woofoo-design-1', 'woofoo-design-2', 'woofoo-design-4', 'woofoo-design-5', 'woofoo-design-7', 'woofoo-design-8', 'woofoo-design-9', 'woofoo-design-10', 'woofoo-design-11'],
                ]
            ]
        );

        //Rating Size.
        $this->add_responsive_control(
			'woofoo_label_spacing', [
				'label'             => esc_html__('Label Spacing', WOOFOO_DOMAIN),
				'type'              => Controls_Manager::SLIDER,
				'size_units'        => [ 'px', '%' ],
				'range'             => [
					'px'            => [
						'min'       => 0,
						'max'       => 100,
						'step'      => 1,
					],
				],
                'default'           => [
                    'size'          => 10,
					'unit'          => 'px',
                ],
				'selectors'         => [
					'{{WRAPPER}} .woofoo-design-9 .woofoo-label' => 'margin-left: {{SIZE}}{{UNIT}};',
				],
                'condition'         => [
                    'woofoo_product_layout' => 'woofoo-design-9',
                ]
			]
		);

        $this->end_controls_section();
        //End Label Style.
        
        //End Style Section
    }

    /**
     * Render WooFoo Elements widget output on the frontend.
     */
    protected function render() {

        $settings = $this->get_settings_for_display();

        $args = array(
            'post_type'             => 'product',
            'posts_per_page'        => $settings['woofoo_number_of_products'],
            'post_status'           => 'publish',
            'orderby'               => $settings['woofoo_product_orderby'],
            'order'                 => $settings['woofoo_product_sort_order'],
        );

        //Product per page.
        if (isset($settings['woofoo_number_of_products']) && !empty($settings['woofoo_number_of_products'])) {
            $args['posts_per_page'] = $settings['woofoo_number_of_products'];
        } else {
            $args['posts_per_page'] = -1;
        }

        //Filter By Product Source.
        $args['meta_query'] = ['relation' => 'AND'];

        if ($settings['apcw_product_source'] == 'featured-products') {
            $args['tax_query'] = [
                'relation'  => 'AND',
                [
                    'taxonomy'  => 'product_visibility',
                    'field'     => 'name',
                    'terms'     => 'featured',
                ],
                [
                    'taxonomy'  => 'product_visibility',
                    'field'     => 'name',
                    'terms'     => ['exclude-from-search', 'exclude-from-catalog'],
                    'operator'  => 'NOT IN',
                ],
            ];
        } else if ($settings['apcw_product_source'] == 'best-selling-products') {
            $args['meta_key']   = 'total_sales';
            $args['orderby']    = 'meta_value_num';
            $args['order']      = 'DESC';
        } else if ($settings['apcw_product_source'] == 'sale-products') {
            $args['post__in']   = array_merge( array( 0 ), wc_get_product_ids_on_sale() );
        } else if ($settings['apcw_product_source'] == 'top-products') {
            $args['meta_key']   = '_wc_average_rating';
            $args['orderby']    = 'meta_value_num';
            $args['order']  = 'DESC';
        }

        //Exclude IN or NOT IN.
        if (isset($settings['woofoo_exclude_product_categories']) && !empty($settings['woofoo_exclude_product_categories'])) {
            $woofoo_product_cats = 'NOT IN';
        } else {
            $woofoo_product_cats = 'IN';
        }
        if (isset($settings['woofoo_exclude_product_tags']) && !empty($settings['woofoo_exclude_product_tags'])) {
            $woofoo_product_tags = 'NOT IN';
        } else {
            $woofoo_product_tags = 'IN';
        }

        //Query Check all Display Category and Tags.
        if ((isset($settings['woofoo_product_categories']) && !empty($settings['woofoo_product_categories'])) && (isset($settings['woofoo_product_tags']) && !empty($settings['woofoo_product_tags']))) {
            //Display All Product with Category and Tags //Condition AND.
            $args['tax_query'][] = array(
                'relation'       => 'AND',
                array(
                    'taxonomy'   => 'product_cat',
                    'field'      => 'ID',
                    'terms'      => $settings['woofoo_product_categories'],
                    'operator'   => $woofoo_product_cats,
                ),
                array(
                    'field'      => 'ID',
                    'taxonomy'   => 'product_tag',
                    'terms'      => $settings['woofoo_product_tags'],
                    'operator'   => $woofoo_product_tags,
                ),
            );
        } elseif (isset($settings['woofoo_product_categories']) && !empty($settings['woofoo_product_categories'])) {
            //Display All Product with Category.
            $args['tax_query'][] = array(
                array(
                    'taxonomy'   => 'product_cat',
                    'field'      => 'ID',
                    'terms'      => $settings['woofoo_product_categories'],
                    'operator'   => $woofoo_product_cats,
                ),
            );
        } elseif (isset($settings['woofoo_product_tags']) && !empty($settings['woofoo_product_tags'])) {
            //Display All Product with Tags. 
            $args['tax_query'][] = array(
                array(
                    'taxonomy'   => 'product_tag',
                    'field'      => 'ID',
                    'terms'      => $settings['woofoo_product_tags'],
                    'operator'   => $woofoo_product_tags,
                ),
            );
        }

        $products = new \WP_Query($args); ?>

        <div class="woofoo-cat-tag-title">
            <?php
            //Product Category Title.
            if($settings['woofoo_product_source'] == "woocommerce") {
                $woofoo_term_ids = $settings['woofoo_product_categories'];
                if (is_array($woofoo_term_ids) || is_object($woofoo_term_ids)){
                    foreach ($woofoo_term_ids as $woofoo_cat_id) {
                        $woofoo_term = get_term_by('id', $woofoo_cat_id, 'product_cat'); ?>
                        <h2 class="woofoo-category">
                            <?php if ($woofoo_term) {
                                echo $woofoo_term->name;
                            } ?>
                        </h2>
                    <?php }
                }
                //Product Tag Title.
                $woofoo_term_ids = $settings['woofoo_product_tags'];
                if (is_array($woofoo_term_ids) || is_object($woofoo_term_ids)){
                    foreach ($woofoo_term_ids as $woofoo_tag_id) {
                        $woofoo_term = get_term_by('id', $woofoo_tag_id, 'product_tag'); ?>
                        <h2 class="woofoo-tag">
                            <?php if ($woofoo_term) {
                                echo $woofoo_term->name;
                            } ?>
                        </h2>
                    <?php }
                } 
            }?>
        </div>
        <div class="woofoo-container <?php echo $settings['woofoo_product_layout'] ?>">
            <?php
            if($settings['woofoo_product_source'] == "woocommerce" && class_exists('WooCommerce')) {
                //For Woocommerce Data.
                while ($products->have_posts()) : $products->the_post();  
                    switch ($settings['woofoo_product_layout']) {
                        case 'woofoo-design-1':
                            include WOOFOO_PATH . 'include/designs/design-1.php'; // Design 1.
                            break;
                        case 'woofoo-design-2':
                            include WOOFOO_PATH . 'include/designs/design-2.php'; // Design 2.
                            break;
                        case 'woofoo-design-3':
                            include WOOFOO_PATH . 'include/designs/design-3.php'; // Design 3.
                            break;
                        case 'woofoo-design-4':
                            include WOOFOO_PATH . 'include/designs/design-4.php'; // Design 4.
                            break;
                        case 'woofoo-design-5':
                            include WOOFOO_PATH . 'include/designs/design-5.php'; // Design 5.
                            break;
                        case 'woofoo-design-6':
                            include WOOFOO_PATH . 'include/designs/design-6.php'; // Design 6.
                            break;
                        case 'woofoo-design-7':
                            include WOOFOO_PATH . 'include/designs/design-7.php'; // Design 7.
                            break;
                        case 'woofoo-design-8':
                            include WOOFOO_PATH . 'include/designs/design-8.php'; // Design 8.
                            break;
                        case 'woofoo-design-9':
                            include WOOFOO_PATH . 'include/designs/design-9.php'; // Design 9.
                            break;
                        case 'woofoo-design-10':
                            include WOOFOO_PATH . 'include/designs/design-10.php'; // Design 10.
                            break;
                        case 'woofoo-design-11':
                            include WOOFOO_PATH . 'include/designs/design-11.php'; // Design 11.
                            break;
                        default:
                            include WOOFOO_PATH . 'include/designs/design-1.php'; // Default Design 1.
                            break;
                    }
                endwhile;
                wp_reset_postdata();
            }
            //For custom Data.
            if($settings['woofoo_product_source'] == "custom") {
                switch ($settings['woofoo_product_layout']) {
                    case 'woofoo-design-1':
                        include WOOFOO_PATH . 'include/designs/design-1.php'; // Design 1.
                        break;
                    case 'woofoo-design-2':
                        include WOOFOO_PATH . 'include/designs/design-2.php'; // Design 2.
                        break;
                    case 'woofoo-design-3':
                        include WOOFOO_PATH . 'include/designs/design-3.php'; // Design 3.
                        break;
                    case 'woofoo-design-4':
                        include WOOFOO_PATH . 'include/designs/design-4.php'; // Design 4.
                        break;
                    case 'woofoo-design-5':
                        include WOOFOO_PATH . 'include/designs/design-5.php'; // Design 5.
                        break;
                    case 'woofoo-design-6':
                        include WOOFOO_PATH . 'include/designs/design-6.php'; // Design 6.
                        break;
                    case 'woofoo-design-7':
                        include WOOFOO_PATH . 'include/designs/design-7.php'; // Design 7.
                        break;
                    case 'woofoo-design-8':
                        include WOOFOO_PATH . 'include/designs/design-8.php'; // Design 8.
                        break;
                    case 'woofoo-design-9':
                        include WOOFOO_PATH . 'include/designs/design-9.php'; // Design 9.
                        break;
                    case 'woofoo-design-10':
                        include WOOFOO_PATH . 'include/designs/design-10.php'; // Design 10.
                        break;
                    case 'woofoo-design-11':
                        include WOOFOO_PATH . 'include/designs/design-11.php'; // Design 11.
                        break;
                    default:
                        include WOOFOO_PATH . 'include/designs/design-1.php'; // Default Design 1.
                        break;
                }
            } ?>
        </div>
        <?php
    }
}

Plugin::instance()->widgets_manager->register(new WOOFOO_Widget());