<?php
/*
 * Plugin Name: WooFoo - Restaurant Food Menu for Elementor
 * Description: WooFoo - Restaurant Food Menu is a spectacular WooCommerce food menu plugin perfect for restaurants, cafés, bars, and more.
 * Plugin URI: https://www.techeshta.com/downloads/woofoo
 * Author: 20script
 * Version: 1.0.3
 * Author URI: https://www.20script.ir
 * Text Domain: woofoo
*/

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

/*
* Define Plugin URL and Directory Path
*/

define('WOOFOO_URL', plugins_url('/', __FILE__));  // Define Plugin URL
define('WOOFOO_PATH', plugin_dir_path(__FILE__));  // Define Plugin Directory Path
define('WOOFOO_DOMAIN', 'woofoo'); // Define Text Domain
define('WOOFOO_SITE', home_url()); 
define('WOOFOO_NAME', 'WooFoo');

/**
 * Main Plugin woofoo class.
 * 
 * @access public
 * @since  1.0
 */
if (!class_exists('WOOFOO')) :
	class WOOFOO {

		/**
		 * WOOFOO constructor.
		 * The main plugin actions registered for WordPress
		 * 
		 * @access public
		 * @since  1.0
		*/
		public function __construct() {
			$this->hooks();
			require_once WOOFOO_PATH . 'widgets/elementor-helper.php';
			require_once WOOFOO_PATH . 'widgets/elementor-dependency.php';
		}

		/**
		 * Initialize
		*/
		public function hooks() {
			// Add Elementor Widgets if plugin activate
			add_action('elementor/widgets/register', array($this, 'woofoo_widget_register'));
			add_action('wp_enqueue_scripts', array($this, 'woofoo_widget_script_register'));
			add_action('plugins_loaded', array($this, 'woofoo_plugin_load'));
		}	

		/**
		 * Register the widgets file in elementor widgets.
		*/
		public function woofoo_widget_register() {
			require_once WOOFOO_PATH . 'widgets/woofoo-widget.php';
			require_once WOOFOO_PATH . 'include/woofoo-functions.php';
		}

		/**
		 * Load scripts and styles
		*/
		public function woofoo_widget_script_register() { 
			wp_enqueue_style('woofoo-style', WOOFOO_URL . 'assets/css/woofoo.css');
		}

		/**
		 * Check for Elementor
		*/
		public function woofoo_plugin_load() {
			// Load plugin textdomain
			load_plugin_textdomain( WOOFOO_DOMAIN, false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );

			// Elementor Install 
			if (!did_action('elementor/loaded')) {
				add_action('admin_notices', array($this, 'woofoo_widget_fail_load'));
				return;
			}
		}

		/**
		 * This notice will appear if Elementor is not installed or activated or both
		*/
		public function woofoo_widget_fail_load() {
			$screen = get_current_screen();
			if (isset($screen->parent_file) && 'plugins.php' === $screen->parent_file && 'update' === $screen->id) {
				return;
			}

			$plugin = 'elementor/elementor.php';

			if (woofoo_elementor_installed()) {
				if (!current_user_can('activate_plugins')) {
					return;
				}
				$activation_url = wp_nonce_url('plugins.php?action=activate&amp;plugin=' . $plugin . '&amp;plugin_status=all&amp;paged=1&amp;s', 'activate-plugin_' . $plugin);

				$message = '<p><strong>' . esc_html__('WooFoo - Restaurant Food Menu for Elementor', WOOFOO_DOMAIN).'</strong>'. esc_html__(' not working because you need to activate the Elementor plugin.', WOOFOO_DOMAIN) . '</p>';
				$message .= '<p>' . sprintf('<a href="%s" class="button-primary">%s</a>', $activation_url, esc_html__('Activate Elementor Now', WOOFOO_DOMAIN)) . '</p>';
			} else {
				if (!current_user_can('install_plugins')) {
					return;
				}

				$install_url = wp_nonce_url(self_admin_url('update.php?action=install-plugin&plugin=elementor'), 'install-plugin_elementor');

				$message = '<p><strong>' . esc_html__('WooFoo - Restaurant Food Menu for Elementor', WOOFOO_DOMAIN).'</strong>'. esc_html__(' not working because you need to install the Elementor plugin', WOOFOO_DOMAIN) . '</p>';
				$message .= '<p>' . sprintf('<a href="%s" class="button-primary">%s</a>', $install_url, esc_html__('Install Elementor Now', WOOFOO_DOMAIN)) . '</p>';
			}

			echo '<div class="error"><p>' . $message . '</p></div>';
		}
	}
endif;

/**
 * Initialize Plugin Class
 */
new WOOFOO();